''===============================================================================
''==   jm_max7219_demo.spin                                                    ==
''==   Demo program using jm_max7219.spin object                               ==
''==   All credit goes to Jon "JonnyMac" McPhalen for his work on the object.  ==
''==   12/08/2013                                                              ==
''===============================================================================


CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000

CON { PINS }

  Clk           = 17            'pin 13 on the MAX7219
  Dout          = 13            'pin 1 on the MAX7219
  Load          = 18            'pin 12 on the MAX7219

CON { Inatial setup for the max7219 }

  Brightness    = 8             ' change to set the inital brightness
  NumbDigits    = 5             ' how many 7-segment displays
  DecodeMode    = %00011111     ' Check page 7, table 4 to know how to set Decode Mode

CON { max7219 registers }

  NO_OP     = $00
  
  DIG_0     = $01
  DIG_1     = $02
  DIG_2     = $03
  DIG_3     = $04
  DIG_4     = $05
  DIG_5     = $06
  DIG_6     = $07
  DIG_7     = $08

  DECODE    = $09
  INTENSITY = $0A
  SCAN_LMT  = $0B
  SHUTDOWN  = $0C

  DISP_TEST = $0F
    
OBJ
  max7219      : "jm_max7219"
  
PUB start

  'Befor using the MAX7219, it must be initialized.
  'First we set the pin the max7219 is hooked to
  max7219.start(Dout, Clk, Load)

  'Now we have to initialize the max7219
  max7219.out(DECODE, DecodeMode)                       ' set the decode mode so digits 0 to 4 are BCD code
  max7219.out(INTENSITY, Brightness)                    ' set brightness of the displays to 8, can be 1 to 15
  max7219.out(SCAN_LMT, NumbDigits - 1)                 ' set number of total digits being used. 
  max7219.out(SHUTDOWN, %00000001)                      ' take the max7219 out of shutdown mode

  Main                                                  ' goto main demo

PUB main | temp, i, x

  'This method will show how to use the jm_max7219.spin object



  '=== display test, this will drive all the digits and segments at full brightness ===
  max7219.out(DISP_TEST, %00000001)                     'turn on display test mode 
  pause(2000)
  max7219.out(DISP_TEST, %00000000)                     'turn off display test mode

  '=== display a value on the 7-segment displays ===
  temp := 12345
  max7219.extract(0, temp)                              'extract temp and place in jm_max7219, segments array
  max7219.Write_buf(0, 5, 1)                            'write segments array to max7219

  pause(2000)

  '=== add decimal point ===
  temp := 54321
  max7219.extract(0, temp)                              'extract new number
  max7219.set_dpoint(0, 2)                              'turn on decimal point of digit 2 
  max7219.Write_buf(0, 5, 1)                            'write to the max7219

  pause(2000)

  '=== clear decimal point ===
  temp := 00000
  max7219.extract(0, temp)                              'extract new number
  max7219.clear_dpoint(0, 2)                            'turn off decimal point of digit 2
  max7219.Write_buf(0, 5, 1)                            'write to the max7219  

  '=== lets move the decimal point around ===
  'move decimal point from digit 1 to digit 5, 3 times
  repeat x from 0 to 3                                  'do it 3 times                                  '
    max7219.extract(0, x)                               'extract x to keep tract of where we are at
    max7219.Write_buf(0, 5, 1)                          'write to the max7219 
    repeat i from 0 to 4                                'what digit are we working with
      max7219.set_dpoint(0, i)                          'set decimal point of digit in i
      max7219.Write_buf(0, 5, 1)                        'write to the max7219
      pause(250)
      max7219.clear_dpoint(0, i)                        'clear the decimal point
      max7219.Write_buf(0, 5, 1)                        'write to the max7219

  pause(500)

  '=== let's count as fast as we can up to 1000 ===
  repeat i from 0 to 1000
    max7219.extract(0, i)                               'extract i
    max7219.Write_buf(0, 5, 1)                          'write to max7219

  pause(500)

  '=== let's show "Error" on the displys ===
  max7219.out(Decode, %00000000)                        'we have to turn off decode mode
  max7219.Write_buf(@error, 5, 1)                       'write to max7219 array with value at error

  pause (500)

  '=== now lets flash the error to get our attention ===
  repeat 10
    max7219.out(shutdown, %00000000)                    'turn off display
    pause (250)
    max7219.out(shutdown, %00000001)                    'turn on display
    pause (250) 


  '=== get a pattern from data, and show it on the displays ===
  repeat i from 250 to 1 step 5
    max7219.Write_buf(@pattern[i], 5, 1)                'show rolling pattern on displays. could be used for an interesting way to check 
    pause(100)                                          '    all displays and leds instead of using Disp_test

    
  '=== lets show a value on the displays again ===
  max7219.out(Decode, DecodeMode)                       'turn decode mode back on
  temp := 7219
  max7219.extract(0, temp)                              'extract new value
  max7219.Write_buf(0, 5, 1)                            'write to the max7219

  pause(500)

  'change the brightness of the displays
  repeat i from 1 to 15
    max7219.out(INTENSITY, i)                           'send max7219 new intensity value
    pause (500)

pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)

DAT
error   byte  %00000101, %00011101, %00000101, %00000101, %01001111        

pattern byte  %00000000, %00000000, %00000000, %00000000, %00000000
        byte  %11000000, %00000000, %00000000, %00000000, %00000000
        byte  %00100000, %00000000, %00000000, %00000000, %00000000
        byte  %10000001, %00000000, %00000000, %00000000, %00000000
        byte  %00010000, %00000000, %00000000, %00000000, %00000000
        byte  %10001000, %00000000, %00000000, %00000000, %00000000
        byte  %00000100, %00000000, %00000000, %00000000, %00000000
        byte  %10000001, %00000000, %00000000, %00000000, %00000000
        byte  %00000010, %00000000, %00000000, %00000000, %00000000
        byte  %00000000, %00000000, %00000000, %00000000, %00000000

        byte  %00000000, %00000000, %00000000, %00000000, %00000000
        byte  %00000000, %11000000, %00000000, %00000000, %00000000
        byte  %00000000, %00100000, %00000000, %00000000, %00000000
        byte  %00000000, %10000001, %00000000, %00000000, %00000000
        byte  %00000000, %00010000, %00000000, %00000000, %00000000
        byte  %00000000, %10001000, %00000000, %00000000, %00000000
        byte  %00000000, %00000100, %00000000, %00000000, %00000000
        byte  %00000000, %10000001, %00000000, %00000000, %00000000
        byte  %00000000, %00000010, %00000000, %00000000, %00000000
        byte  %00000000, %00000000, %00000000, %00000000, %00000000

        byte  %00000000, %00000000, %00000000, %00000000, %00000000
        byte  %00000000, %00000000, %11000000, %00000000, %00000000
        byte  %00000000, %00000000, %00100000, %00000000, %00000000
        byte  %00000000, %00000000, %10000001, %00000000, %00000000
        byte  %00000000, %00000000, %00010000, %00000000, %00000000
        byte  %00000000, %00000000, %10001000, %00000000, %00000000
        byte  %00000000, %00000000, %00000100, %00000000, %00000000
        byte  %00000000, %00000000, %10000001, %00000000, %00000000
        byte  %00000000, %00000000, %00000010, %00000000, %00000000
        byte  %00000000, %00000000, %00000000, %00000000, %00000000

        byte  %00000000, %00000000, %00000000, %00000000, %00000000
        byte  %00000000, %00000000, %00000000, %11000000, %00000000
        byte  %00000000, %00000000, %00000000, %00100000, %00000000
        byte  %00000000, %00000000, %00000000, %10000001, %00000000
        byte  %00000000, %00000000, %00000000, %00010000, %00000000
        byte  %00000000, %00000000, %00000000, %10001000, %00000000
        byte  %00000000, %00000000, %00000000, %00000100, %00000000
        byte  %00000000, %00000000, %00000000, %10000001, %00000000
        byte  %00000000, %00000000, %00000000, %00000010, %00000000
        byte  %00000000, %00000000, %00000000, %00000000, %00000000

        byte  %00000000, %00000000, %00000000, %00000000, %00000000
        byte  %00000000, %00000000, %00000000, %00000000, %11000000
        byte  %00000000, %00000000, %00000000, %00000000, %00100000
        byte  %00000000, %00000000, %00000000, %00000000, %10000001
        byte  %00000000, %00000000, %00000000, %00000000, %00010000
        byte  %00000000, %00000000, %00000000, %00000000, %10001000
        byte  %00000000, %00000000, %00000000, %00000000, %00000100
        byte  %00000000, %00000000, %00000000, %00000000, %10000001
        byte  %00000000, %00000000, %00000000, %00000000, %00000010
        byte  %00000000, %00000000, %00000000, %00000000, %00000000

        

         

        